# SPDX-FileCopyrightText: 2023 Wolfgang Langner <tds333@mailbox.org>
#
# SPDX-License-Identifier: Apache-2.0 OR MIT

import sys
import traceback
from io import StringIO
from types import FrameType


def get_frame_fallback(n) -> FrameType:
    try:
        raise Exception
    except Exception:
        frame = sys.exc_info()[2].tb_frame.f_back  # type: ignore
        for _ in range(n):
            frame = frame.f_back  # type: ignore
        return frame  # type: ignore


def load_get_frame_function():
    if hasattr(sys, "_getframe"):
        get_frame = sys._getframe
    else:
        get_frame = get_frame_fallback
    return get_frame


get_frame = load_get_frame_function()


def _format_exception(exc_info):
    """
    Prettyprint an `exc_info` tuple.

    Shamelessly stolen from stdlib's logging module.
    """
    sio = StringIO()

    traceback.print_exception(exc_info[0], exc_info[1], exc_info[2], None, sio)
    s = sio.getvalue()
    sio.close()
    if s[-1:] == "\n":
        s = s[:-1]

    return s
